import path from 'path'
import { fileURLToPath } from 'url'
import { importFromString, importFromStringSync } from '../../src/index'

const __dirname = path.dirname(fileURLToPath(import.meta.url))

describe('importFromString', () => {
  it('should work with named export', async () => {
    const res = await importFromString("export const greet = 'hi'")
    expect(res.greet).toBe('hi')
  })

  it('should work with default export', async () => {
    const res = await importFromString("export default 'hi'")
    expect(res.default).toBe('hi')
  })

  it('should work with relative path import', async () => {
    const modulePath = './fixtures/namedExport.js'
    const res = await importFromString(`export { greet } from '${modulePath}'`)
    expect(res.greet).toBe('hi')
  })

  it('should resolve correctly if option `dirname` is provided', async () => {
    const modulePath = './esm/fixtures/defaultExport.js'
    const res = await importFromString(`export { default } from '${modulePath}'`, {
      dirname: path.dirname(__dirname)
    })
    expect(res.default).toBe('hi')
  })

  it('should work with absolute path import', async () => {
    const modulePath = path.join(__dirname, 'fixtures/namedExport.js')
    const res = await importFromString(`export { greet } from '${modulePath}'`)
    expect(res.greet).toBe('hi')
  })

  it('should work with import external module', async () => {
    const code = `import { transformSync } from 'esbuild'
const { code } = transformSync('enum Greet { Hi }', { loader: 'ts' })
export default code
`
    const res = await importFromString(code)
    expect(res.default).toMatchInlineSnapshot(`
      "var Greet = /* @__PURE__ */ ((Greet2) => {
        Greet2[Greet2[\\"Hi\\"] = 0] = \\"Hi\\";
        return Greet2;
      })(Greet || {});
      "
    `)
  })

  it('should be able to use dynamic import', async () => {
    const modulePath = './fixtures/namedExport.js'
    const res = await importFromString(`export const greetModule = import('${modulePath}')`)
    expect((await res.greetModule).greet).toBe('hi')
  })

  it('should be able to access __dirname and __filename', () => {
    const res = importFromStringSync(`
      export const dirname = __dirname
      export const filename = __filename
    `)
    expect(res.dirname).toBe(__dirname)
    expect(res.filename).toMatch(__dirname)
  })

  it('should be able to access import.meta.url', async () => {
    const res = await importFromString('export const { url } = import.meta')
    expect(res.url).toMatch(`file://${__dirname}`)
  })

  it('should not access other globals', async () => {
    expect.assertions(1)
    // eslint-disable-next-line @typescript-eslint/promise-function-async
    const res = (): Promise<string> => importFromString('export default process.cwd()')
    try {
      await res()
    } catch (err) {
      expect(Object.getPrototypeOf(err)).toHaveProperty('name', 'ReferenceError')
    }
  })

  it('should work with provided globals', async () => {
    const res = await importFromString('export const cwd = process.cwd()', {
      globals: { process }
    })
    expect(res.cwd).toBe(process.cwd())
  })

  it('should have access the global object', async () => {
    const res = await importFromString('export const { greet } = global', {
      globals: { greet: 'hi' }
    })
    expect(res.greet).toBe('hi')
  })

  it('should have same globalThis', async () => {
    const res = await importFromString('export default global === globalThis')
    expect(res.default).toBeTruthy()
  })

  it('should work with current global', async () => {
    const res = await importFromString('export default new Error()', {
      useCurrentGlobal: true
    })
    expect(res.default).toBeInstanceOf(Error)
  })

  it('should work if transformOption is provided', async () => {
    const res = await importFromString("export default function(): string { return 'hi' }", {
      transformOptions: { loader: 'ts' }
    })
    expect(res.default()).toBe('hi')
  })
})

describe('importFromStringSync', () => {
  it('should throw error if import ES module', () => {
    const modulePath = './fixtures/defaultExport.js'
    expect(() => {
      importFromStringSync(`export { default } from '${modulePath}'`)
    }).toThrowError(
      `'import' statement of ES modules is not supported
Use asynchronous function 'importFromString' instead or replace it with dynamic 'import()' expression.`
    )
  })
})
